package test.uitest;

import static org.fest.swing.data.TableCell.row;
import static org.fest.swing.finder.WindowFinder.findFrame;
import static org.fest.swing.launcher.ApplicationLauncher.application;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Properties;
import java.awt.Point;
import java.awt.event.KeyEvent;

import javax.swing.JFrame;

import org.fest.swing.core.BasicRobot;
import org.fest.swing.core.GenericTypeMatcher;
import org.fest.swing.core.Robot;
import org.fest.swing.edt.FailOnThreadViolationRepaintManager;
import org.fest.swing.finder.JFileChooserFinder;
import org.fest.swing.fixture.DialogFixture;
import org.fest.swing.fixture.FrameFixture;
import org.fest.swing.fixture.JButtonFixture;
import org.fest.swing.fixture.JFileChooserFixture;
import org.fest.swing.fixture.JMenuItemFixture;
import org.fest.swing.fixture.JTableCellFixture;
import org.fest.swing.fixture.JTableFixture;
import org.fest.swing.fixture.JTreeFixture;

import org.junit.After;
import org.junit.Before;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TestName;
import org.junit.Ignore;

import hdf.HDFVersions;

public class TestHDFViewDatasetFrameSelection {
    @Rule public TestName testname = new TestName();
    private static FrameFixture mainFrameFixture;
    // the version of the HDFViewer
    private static String VERSION = HDFVersions.HDFVIEW_VERSION;
    private static String workDir = System.getProperty("hdfview.workdir");

    private File openHDF5File(String name) {
        File hdf_file = new File(workDir, name+".h5");

        try {
            JMenuItemFixture fileMenuItem = mainFrameFixture.menuItemWithPath("File","Open");
            mainFrameFixture.robot.waitForIdle();

            fileMenuItem.requireVisible();
            fileMenuItem.click();
            mainFrameFixture.robot.waitForIdle();

            JFileChooserFixture fileChooser = JFileChooserFinder.findFileChooser().using(mainFrameFixture.robot);
            fileChooser.fileNameTextBox().setText(name+".h5");
            fileChooser.approve();
            mainFrameFixture.robot.waitForIdle();

            assertTrue("HDF5 File- " + hdf_file + "not opened", hdf_file.exists());
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        catch (AssertionError ae) {
            ae.printStackTrace();
        }

        return hdf_file;
    }

    private void closeHDFFile(File hdf_file, boolean delete_file) {
        try {
            JMenuItemFixture fileMenuItem = mainFrameFixture.menuItemWithPath("File","Close All");
            mainFrameFixture.robot.waitForIdle();

            fileMenuItem.requireVisible();
            fileMenuItem.click();
            mainFrameFixture.robot.waitForIdle();

            if(delete_file) {
                assertTrue("File " + hdf_file + "not deleted", hdf_file.delete());
                assertFalse("File " + hdf_file + "not gone", hdf_file.exists());
            }

            JTreeFixture filetree = mainFrameFixture.tree().focus();
            assertTrue("closeHDFFile filetree shows:"+filetree.target.getRowCount(), filetree.target.getRowCount() == 0);
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        catch (AssertionError ae) {
            ae.printStackTrace();
        }
    }

    private static void clearRemovePropertyFile() {
        // the local property file name
        // look for the property file at the use home directory
        String fn = ".hdfview" + VERSION;

        File prop_file = new File(workDir, fn);
        if (prop_file.exists()) {
            prop_file.delete();
        }
    }

    @BeforeClass
    public static void setUpOnce() {
        clearRemovePropertyFile();
        FailOnThreadViolationRepaintManager.install();
        Robot robot = BasicRobot.robotWithNewAwtHierarchy();
        application("hdf.view.HDFView").withArgs("-root", workDir).start();
        mainFrameFixture = findFrame(new GenericTypeMatcher<JFrame>(JFrame.class) {
            protected boolean isMatching(JFrame frame) {
                return frame.getTitle().equals("HDFView "+VERSION)
                        && frame.isShowing();
            }
        }).withTimeout(10000).using(robot);
        mainFrameFixture.robot.waitForIdle();
        mainFrameFixture.requireVisible();
    }

    @AfterClass
    public static void finishApplication() {
        System.out.println("finished");
        mainFrameFixture.robot.waitForIdle();
        mainFrameFixture.cleanUp();
    }

    @Before
    public void setTestName() {
        System.out.print(testname.getMethodName());
    }

    @After
    public void setTestResult() {
        System.out.println();
    }

    @Test
    public void testNextFrame() {
        File hdf_file = openHDF5File("tframeselection");

        try {
            JTreeFixture filetree = mainFrameFixture.tree().focus();
            filetree.requireVisible();
            assertTrue("testNextFrame filetree shows: "+filetree.target.getRowCount(), filetree.target.getRowCount()==2);
            assertTrue("testNextFrame filetree has file "+filetree.valueAt(0), (filetree.valueAt(0)).compareTo("tframeselection.h5")==0);
            assertTrue("testNextFrame filetree has dataset "+filetree.valueAt(1), (filetree.valueAt(1)).compareTo("test_dataset")==0);

            JMenuItemFixture dataset1MenuItem = filetree.showPopupMenuAt(1).menuItemWithPath("Open");
            mainFrameFixture.robot.waitForIdle();

            dataset1MenuItem.requireVisible();
            dataset1MenuItem.click();
            mainFrameFixture.robot.waitForIdle();

            JTableFixture tblFix = mainFrameFixture.table("data");

            try {
                mainFrameFixture.textBox("framenumber").requireText("0");
            }
            catch (AssertionError ae) {
                mainFrameFixture.textBox("framenumber").deleteText();
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").enterText("0");
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").pressAndReleaseKeys(10);
                mainFrameFixture.robot.waitForIdle();
            }

            JTableCellFixture cellFix = tblFix.cell(row(4).column(2));
            cellFix.requireValue("478");

            cellFix = tblFix.cell(row(1).column(3));
            cellFix.requireValue("52");

            mainFrameFixture.button("nextbutton").click();
            mainFrameFixture.robot.waitForIdle();

            mainFrameFixture.textBox("framenumber").requireText("1");

            cellFix = tblFix.cell(row(3).column(4));
            cellFix.requireValue("454");

            cellFix = tblFix.cell(row(2).column(1));
            cellFix.requireValue("984");

            mainFrameFixture.menuItemWithPath("Table", "Close").click();
            mainFrameFixture.robot.waitForIdle();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        catch (AssertionError ae) {
            ae.printStackTrace();
        }
        finally {
            try {
                closeHDFFile(hdf_file, false);
            }
            catch (Exception ex) {}
        }
    }

    @Test
    public void testPreviousFrame() {
        File hdf_file = openHDF5File("tframeselection");

        try {
            JTreeFixture filetree = mainFrameFixture.tree().focus();
            filetree.requireVisible();
            assertTrue("testNextFrame filetree shows: "+filetree.target.getRowCount(), filetree.target.getRowCount()==2);
            assertTrue("testNextFrame filetree has file "+filetree.valueAt(0), (filetree.valueAt(0)).compareTo("tframeselection.h5")==0);
            assertTrue("testNextFrame filetree has dataset "+filetree.valueAt(1), (filetree.valueAt(1)).compareTo("test_dataset")==0);

            JMenuItemFixture dataset1MenuItem = filetree.showPopupMenuAt(1).menuItemWithPath("Open");
            mainFrameFixture.robot.waitForIdle();

            dataset1MenuItem.requireVisible();
            dataset1MenuItem.click();
            mainFrameFixture.robot.waitForIdle();

            JTableFixture tblFix = mainFrameFixture.table("data");

            try {
                mainFrameFixture.textBox("framenumber").requireText("1");
            } catch (AssertionError ae) {
                mainFrameFixture.textBox("framenumber").deleteText();
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").enterText("1");
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").pressAndReleaseKeys(10);
                mainFrameFixture.robot.waitForIdle();
            }

            JTableCellFixture cellFix = tblFix.cell(row(3).column(1));
            cellFix.requireValue("6");

            cellFix = tblFix.cell(row(4).column(2));
            cellFix.requireValue("215");

            mainFrameFixture.button("prevbutton").click();
            mainFrameFixture.robot.waitForIdle();

            mainFrameFixture.textBox("framenumber").requireText("0");

            cellFix = tblFix.cell(row(0).column(0));
            cellFix.requireValue("13");

            cellFix = tblFix.cell(row(4).column(4));
            cellFix.requireValue("4");

            mainFrameFixture.menuItemWithPath("Table", "Close").click();
            mainFrameFixture.robot.waitForIdle();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        catch (AssertionError ae) {
            ae.printStackTrace();
        }
        finally {
            try {
                closeHDFFile(hdf_file, false);
            }
            catch (Exception ex) {}
        }
    }

    @Test
    public void testFirstFrame() {
        File hdf_file = openHDF5File("tframeselection");

        try {
            JTreeFixture filetree = mainFrameFixture.tree().focus();
            filetree.requireVisible();
            assertTrue("testNextFrame filetree shows: "+filetree.target.getRowCount(), filetree.target.getRowCount()==2);
            assertTrue("testNextFrame filetree has file "+filetree.valueAt(0), (filetree.valueAt(0)).compareTo("tframeselection.h5")==0);
            assertTrue("testNextFrame filetree has dataset "+filetree.valueAt(1), (filetree.valueAt(1)).compareTo("test_dataset")==0);

            JMenuItemFixture dataset1MenuItem = filetree.showPopupMenuAt(1).menuItemWithPath("Open");
            mainFrameFixture.robot.waitForIdle();

            dataset1MenuItem.requireVisible();
            dataset1MenuItem.click();
            mainFrameFixture.robot.waitForIdle();

            JTableFixture tblFix = mainFrameFixture.table("data");

            try {
                mainFrameFixture.textBox("framenumber").requireText("3");
            } catch (AssertionError ae) {
                mainFrameFixture.textBox("framenumber").deleteText();
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").enterText("3");
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").pressAndReleaseKeys(10);
                mainFrameFixture.robot.waitForIdle();
            }

            JTableCellFixture cellFix = tblFix.cell(row(3).column(1));
            cellFix.requireValue("456");

            cellFix = tblFix.cell(row(0).column(2));
            cellFix.requireValue("7");

            mainFrameFixture.button("firstbutton").click();
            mainFrameFixture.robot.waitForIdle();

            mainFrameFixture.textBox("framenumber").requireText("0");

            cellFix = tblFix.cell(row(3).column(4));
            cellFix.requireValue("52");

            cellFix = tblFix.cell(row(4).column(1));
            cellFix.requireValue("345");

            mainFrameFixture.menuItemWithPath("Table", "Close").click();
            mainFrameFixture.robot.waitForIdle();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        catch (AssertionError ae) {
            ae.printStackTrace();
        }
        finally {
            try {
                closeHDFFile(hdf_file, false);
            }
            catch (Exception ex) {}
        }
    }

    @Test
    public void testLastFrame() {
        File hdf_file = openHDF5File("tframeselection");

        try {
            JTreeFixture filetree = mainFrameFixture.tree().focus();
            filetree.requireVisible();
            assertTrue("testNextFrame filetree shows: "+filetree.target.getRowCount(), filetree.target.getRowCount()==2);
            assertTrue("testNextFrame filetree has file "+filetree.valueAt(0), (filetree.valueAt(0)).compareTo("tframeselection.h5")==0);
            assertTrue("testNextFrame filetree has dataset "+filetree.valueAt(1), (filetree.valueAt(1)).compareTo("test_dataset")==0);

            JMenuItemFixture dataset1MenuItem = filetree.showPopupMenuAt(1).menuItemWithPath("Open");
            mainFrameFixture.robot.waitForIdle();

            dataset1MenuItem.requireVisible();
            dataset1MenuItem.click();
            mainFrameFixture.robot.waitForIdle();

            JTableFixture tblFix = mainFrameFixture.table("data");

            try {
                mainFrameFixture.textBox("framenumber").requireText("0");
            } catch (AssertionError ae) {
                mainFrameFixture.textBox("framenumber").deleteText();
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").enterText("0");
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").pressAndReleaseKeys(10);
                mainFrameFixture.robot.waitForIdle();
            }

            JTableCellFixture cellFix = tblFix.cell(row(2).column(3));
            cellFix.requireValue("63");

            cellFix = tblFix.cell(row(1).column(0));
            cellFix.requireValue("2");

            mainFrameFixture.button("lastbutton").click();
            mainFrameFixture.robot.waitForIdle();

            mainFrameFixture.textBox("framenumber").requireText("4");

            cellFix = tblFix.cell(row(0).column(4));
            cellFix.requireValue("789");

            cellFix = tblFix.cell(row(4).column(1));
            cellFix.requireValue("7945");

            mainFrameFixture.menuItemWithPath("Table", "Close").click();
            mainFrameFixture.robot.waitForIdle();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        catch (AssertionError ae) {
            ae.printStackTrace();
        }
        finally {
            try {
                closeHDFFile(hdf_file, false);
            }
            catch (Exception ex) {}
        }
    }

    @Test
    public void testEnterFrame() {
        File hdf_file = openHDF5File("tframeselection");

        try {
            JTreeFixture filetree = mainFrameFixture.tree().focus();
            filetree.requireVisible();
            assertTrue("testNextFrame filetree shows: "+filetree.target.getRowCount(), filetree.target.getRowCount()==2);
            assertTrue("testNextFrame filetree has file "+filetree.valueAt(0), (filetree.valueAt(0)).compareTo("tframeselection.h5")==0);
            assertTrue("testNextFrame filetree has dataset "+filetree.valueAt(1), (filetree.valueAt(1)).compareTo("test_dataset")==0);

            JMenuItemFixture dataset1MenuItem = filetree.showPopupMenuAt(1).menuItemWithPath("Open");
            mainFrameFixture.robot.waitForIdle();

            dataset1MenuItem.requireVisible();
            dataset1MenuItem.click();
            mainFrameFixture.robot.waitForIdle();

            JTableFixture tblFix = mainFrameFixture.table("data");

            try {
                mainFrameFixture.textBox("framenumber").requireText("0");
            } catch (AssertionError ae) {
                mainFrameFixture.textBox("framenumber").deleteText();
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").enterText("0");
                mainFrameFixture.robot.waitForIdle();

                mainFrameFixture.textBox("framenumber").pressAndReleaseKeys(10);
                mainFrameFixture.robot.waitForIdle();
            }

            JTableCellFixture cellFix = tblFix.cell(row(3).column(2));
            cellFix.requireValue("99");

            cellFix = tblFix.cell(row(4).column(3));
            cellFix.requireValue("86");

            mainFrameFixture.textBox("framenumber").deleteText();
            mainFrameFixture.robot.waitForIdle();

            mainFrameFixture.textBox("framenumber").enterText("3");
            mainFrameFixture.robot.waitForIdle();

            mainFrameFixture.textBox("framenumber").pressAndReleaseKeys(10);
            mainFrameFixture.robot.waitForIdle();

            cellFix = tblFix.cell(row(2).column(1));
            cellFix.requireValue("63");

            cellFix = tblFix.cell(row(0).column(2));
            cellFix.requireValue("7");

            mainFrameFixture.textBox("framenumber").deleteText();
            mainFrameFixture.robot.waitForIdle();

            mainFrameFixture.textBox("framenumber").enterText("2");
            mainFrameFixture.robot.waitForIdle();

            mainFrameFixture.textBox("framenumber").pressAndReleaseKeys(10);
            mainFrameFixture.robot.waitForIdle();

            cellFix = tblFix.cell(row(2).column(1));
            cellFix.requireValue("88");

            cellFix = tblFix.cell(row(0).column(2));
            cellFix.requireValue("66");

            mainFrameFixture.menuItemWithPath("Table", "Close").click();
            mainFrameFixture.robot.waitForIdle();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        catch (AssertionError ae) {
            ae.printStackTrace();
        }
        finally {
            try {
                closeHDFFile(hdf_file, false);
            }
            catch (Exception ex) {}
        }
    }
}